/* $Id: config.c,v 1.3 2006/11/19 20:12:12 jirka Exp $ */
/*
 * Plugin configuration.
 * Copyright (C) 2005 Jiri Denemark
 *
 * This file is part of gkrellm-tz.
 *
 * gkrellm-tz is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/** @file
 * Plugin configuration.
 * @author Jiri Denemark
 * @date 2005
 * @version $Revision: 1.3 $ $Date: 2006/11/19 20:12:12 $
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <time.h>

#include <glib.h>
#include <gtk/gtk.h>
#include <gkrellm2/gkrellm.h>

#include "list.h"
#include "config.h"

static gchar about_text[] =
    "gkrellm-tz " VERSION "\n"
    "GKrellM Timezone Plugin\n"
    "\n"
    "Copyright (C) 2005--2006 Jiri Denemark <jirka@ics.muni.cz>\n"
    "http://www.fi.muni.cz/~xdenemar/gkrellm/\n"
    "\n"
    "Released under the GNU General Public Licence";

static gchar *info_text[] = {
    "<b>GKrellM Timezone Plugin\n",
    "\n",
    "This plugin displays current time in several configurable timezones.\n",
    "\n",
    "<b>Checkbox\n",
    "\tSpecifies whether current time for a particular timezone will actually be shown or not.\n",
    "<b>Label\n",
    "\tA short description of a timezone. It is shown in a tooltip.\n",
    "<b>Timezone\n",
    "\tTimezone identification as can be found under /usr/share/zoneinfo/.\n",
    "\tFor example, \"Europe/Prague\" or \"UTC\"\n",
    "\n",
    "Configured timezones are stored in ~/.gkrellm2/data/gkrellm-tz file.\n"
};


static gchar *list_titles[] = { "", "Label", "Timezone" };

static GtkWidget *entry_label;
static GtkWidget *entry_tz;
static GtkTreeSelection *sel_row;
static GtkTreeIter sel_row_iter;
static GtkListStore *list_store;
static GtkTreeModel *treemodel;

static void reset_entries(void);
static void config_toggled(GtkCellRendererToggle *cell_renderer,
                           gchar *path,
                           gpointer user_data);
static void config_selected(GtkTreeSelection *selection, gpointer data);
static void config_add(GtkWidget *widget, gpointer data);
static void config_set(GtkWidget *widget, gpointer data);
static void config_delete(GtkWidget *widget, gpointer data);
static void config_up(GtkWidget *widget, gpointer data);
static void config_down(GtkWidget *widget, gpointer data);


void config_list(struct tz_plugin *plugin)
{
    GtkTreeIter iter;
    gboolean enabled;
    gchar *entry[2];
    struct tz_list_item *item;

    if (gtk_tree_model_get_iter_first(treemodel, &iter) == FALSE)
        return;

    do {
        gtk_tree_model_get(treemodel, &iter,
                           0, &enabled,
                           1, &entry[0],
                           2, &entry[1], -1);
        tz_list_add(plugin, enabled == TRUE, entry[0], entry[1]);
    } while (gtk_tree_model_iter_next(treemodel, &iter) == TRUE);

    gtk_list_store_clear(list_store);
    for (item = plugin->first; item != NULL; item = item->next) {
        if (item->tz.enabled)
            enabled = TRUE;
        else
            enabled = FALSE;
        entry[0] = item->tz.label;
        entry[1] = item->tz.timezone;
        gtk_list_store_append(list_store, &iter);
        gtk_list_store_set(list_store, &iter,
                           0, enabled,
                           1, entry[0],
                           2, entry[1], -1);
    }

}


static void config_tz_list(GtkWidget *vbox, struct tz_plugin *plugin)
{
    GtkWidget *scrolled;
    GtkWidget *tree;
    GtkTreeIter iter;
    GtkCellRenderer *renderer;
    GtkTreeViewColumn *column;
    GtkTreeSelection *select;
    gboolean enabled;
    gchar *buf[2];
    struct tz_list_item *item;

    scrolled = gtk_scrolled_window_new(NULL, NULL);
    gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrolled),
            GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
    gtk_box_pack_start(GTK_BOX(vbox), scrolled, TRUE, TRUE, 0);

    list_store = gtk_list_store_new(3, G_TYPE_BOOLEAN, G_TYPE_STRING, G_TYPE_STRING);
    for (item = plugin->first; item != NULL; item = item->next) {
        if (item->tz.enabled)
            enabled = TRUE;
        else
            enabled = FALSE;
        buf[0] = item->tz.label;
        buf[1] = item->tz.timezone;
        gtk_list_store_append(list_store, &iter);
        gtk_list_store_set(list_store, &iter,
                           0, enabled,
                           1, buf[0],
                           2, buf[1], -1);
    }

    treemodel = GTK_TREE_MODEL(list_store);
    tree = gtk_tree_view_new_with_model(treemodel);

    renderer = gtk_cell_renderer_toggle_new();
    column = gtk_tree_view_column_new_with_attributes(list_titles[0],
                    renderer, "active", 0, NULL);
    gtk_tree_view_append_column(GTK_TREE_VIEW(tree), column);
    g_signal_connect(G_OBJECT(renderer), "toggled",
                     G_CALLBACK(config_toggled), NULL);

    renderer = gtk_cell_renderer_text_new();
    column = gtk_tree_view_column_new_with_attributes(list_titles[1],
                    renderer, "text", 1, NULL);
    gtk_tree_view_append_column(GTK_TREE_VIEW(tree), column);

    renderer = gtk_cell_renderer_text_new();
    column = gtk_tree_view_column_new_with_attributes(list_titles[2],
                    renderer, "text", 2, NULL);
    gtk_tree_view_append_column(GTK_TREE_VIEW(tree), column);

    select = gtk_tree_view_get_selection(GTK_TREE_VIEW(tree));
    gtk_tree_selection_set_mode(select, GTK_SELECTION_SINGLE);
    g_signal_connect(G_OBJECT(select), "changed",
                     G_CALLBACK(config_selected), NULL);

    gtk_container_add(GTK_CONTAINER(scrolled), tree);
}


static void config_timezones(GtkWidget *vbox, struct tz_plugin *plugin)
{
    GtkWidget *label;
    GtkWidget *hbox;
    GtkWidget *button;
    GtkWidget *table;

    table = gtk_table_new(2, 2, FALSE);
    gtk_box_pack_start(GTK_BOX(vbox), table, FALSE, TRUE, 0);

    /* Label */
    label = gtk_label_new("Label");
    gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);
    gtk_table_attach_defaults(GTK_TABLE(table), label, 0, 1, 0, 1);
    entry_label = gtk_entry_new_with_max_length(MAX_LABEL_LENGTH);
    gtk_table_attach_defaults(GTK_TABLE(table), entry_label, 1, 2, 0, 1);

    /* Timezone */
    label = gtk_label_new("Timezone");
    gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);
    gtk_table_attach_defaults(GTK_TABLE(table), label, 0, 1, 1, 2);
    entry_tz = gtk_entry_new_with_max_length(MAX_TIMEZONE_LENGTH);
    gtk_table_attach_defaults(GTK_TABLE(table), entry_tz, 1, 2, 1, 2);

    /* Action buttons */
    hbox = gtk_hbutton_box_new();
    gtk_button_box_set_layout(GTK_BUTTON_BOX(hbox), GTK_BUTTONBOX_START);
    gtk_button_box_set_spacing(GTK_BUTTON_BOX(hbox), 5);
    gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, TRUE, 5);

    button = gtk_button_new_from_stock(GTK_STOCK_ADD);
    g_signal_connect(G_OBJECT(button), "clicked",
                       G_CALLBACK(config_add), NULL);
    gtk_container_add(GTK_CONTAINER(hbox), button);

    button = gtk_button_new_from_stock(GTK_STOCK_EDIT);
    g_signal_connect(G_OBJECT(button), "clicked",
                       G_CALLBACK(config_set), NULL);
    gtk_container_add(GTK_CONTAINER(hbox), button);

    button = gtk_button_new_from_stock(GTK_STOCK_DELETE);
    g_signal_connect(G_OBJECT(button), "clicked",
                     G_CALLBACK(config_delete), NULL);
    gtk_container_add(GTK_CONTAINER(hbox), button);

    hbox = gtk_hbox_new(FALSE, 0);
    gtk_box_pack_start(GTK_BOX(vbox), hbox, TRUE, TRUE, 0);

    vbox = gtk_vbox_new(FALSE, 0);
    gtk_box_pack_start(GTK_BOX(hbox), vbox, TRUE, TRUE, 0);
    config_tz_list(vbox, plugin);

    vbox = gtk_vbutton_box_new();
    gtk_button_box_set_layout(GTK_BUTTON_BOX(vbox), GTK_BUTTONBOX_START);
    gtk_button_box_set_spacing(GTK_BUTTON_BOX(vbox), 5);
    gtk_box_pack_start(GTK_BOX(hbox), vbox, FALSE, TRUE, 5);

    button = gtk_button_new_from_stock(GTK_STOCK_GO_UP);
    g_signal_connect(G_OBJECT(button), "clicked",
                     G_CALLBACK(config_up), NULL);
    gtk_container_add(GTK_CONTAINER(vbox), button);

    button = gtk_button_new_from_stock(GTK_STOCK_GO_DOWN);
    g_signal_connect(G_OBJECT(button), "clicked",
                     G_CALLBACK(config_down), NULL);
    gtk_container_add(GTK_CONTAINER(vbox), button);
}


void config_create_tabs(GtkWidget *tab_vbox, struct tz_plugin *plugin)
{
    int i;
    GtkWidget *tabs;
    GtkWidget *label;
    GtkWidget *vbox;
    GtkWidget *text;

    tabs = gtk_notebook_new();
    gtk_notebook_set_tab_pos(GTK_NOTEBOOK(tabs), GTK_POS_TOP);
    gtk_box_pack_start(GTK_BOX(tab_vbox), tabs, TRUE, TRUE, 0);

    /* Timezones configuration */
    vbox = gkrellm_gtk_framed_notebook_page(tabs, "Timezones");
    config_timezones(vbox, plugin);

    /* Info tab */
    vbox = gkrellm_gtk_framed_notebook_page(tabs, "Info");
    text = gkrellm_gtk_scrolled_text_view(vbox, NULL, GTK_POLICY_AUTOMATIC,
                                          GTK_POLICY_AUTOMATIC);
    for (i = 0; i < sizeof(info_text) / sizeof(gchar *); i++)
        gkrellm_gtk_text_view_append(text, info_text[i]);

    /* About tab */
    label = gtk_label_new("About");
    gtk_notebook_append_page(GTK_NOTEBOOK(tabs),
                             gtk_label_new(about_text), label);
}


static void reset_entries(void)
{
    gtk_entry_set_text(GTK_ENTRY(entry_label), "");
    gtk_entry_set_text(GTK_ENTRY(entry_tz), "");
}


static void config_toggled(GtkCellRendererToggle *cell_renderer,
                           gchar *path,
                           gpointer user_data)
{
    gboolean enabled;
    GtkTreeIter iter;

    gtk_tree_model_get_iter_from_string(treemodel, &iter, path);
    gtk_tree_model_get(treemodel, &iter, 0, &enabled, -1);
    gtk_list_store_set(list_store, &iter, 0, !enabled, -1);
}


static void config_selected(GtkTreeSelection *selection, gpointer data)
{
    GtkTreeModel *treemodel;
    GtkTreeIter iter;
    gchar *entry[2];

    if (gtk_tree_selection_get_selected(selection, &treemodel, &iter)) {
        gtk_tree_model_get(treemodel, &iter, 1, &entry[0], 2, &entry[1], -1);
        gtk_entry_set_text(GTK_ENTRY(entry_label), entry[0]);
        gtk_entry_set_text(GTK_ENTRY(entry_tz), entry[1]);
        g_free(entry[0]);
        g_free(entry[1]);

        sel_row_iter = iter;
        sel_row = selection;
    }
    else
        reset_entries();
}


static void config_delete(GtkWidget *widget, gpointer data)
{
    reset_entries();
    gtk_list_store_remove(list_store, &sel_row_iter);
}


static void config_add(GtkWidget *widget, gpointer data)
{
    gchar *entry[2];

    entry[0] = gkrellm_gtk_entry_get_text(&entry_label);
    g_strstrip(entry[0]);
    entry[1] = gkrellm_gtk_entry_get_text(&entry_tz);
    g_strstrip(entry[1]);

    if (strlen(entry[0]) == 0 || strlen(entry[1]) == 0) {
        reset_entries();
        return;
    }

    gtk_list_store_append(list_store, &sel_row_iter);
    gtk_list_store_set(list_store, &sel_row_iter,
                       0, TRUE,
                       1, entry[0],
                       2, entry[1], -1);

    reset_entries();
}


static void config_set(GtkWidget *widget, gpointer data)
{
    gchar *entry[2];

    entry[0] = gkrellm_gtk_entry_get_text(&entry_label);
    g_strstrip(entry[0]);
    entry[1] = gkrellm_gtk_entry_get_text(&entry_tz);
    g_strstrip(entry[1]);

    if (strlen(entry[0]) == 0 || strlen(entry[1]) == 0) {
        reset_entries();
        return;
    }

    gtk_list_store_set(list_store, &sel_row_iter,
                       0, TRUE,
                       1, entry[0],
                       2, entry[1], -1);

    reset_entries();
}


static void config_up(GtkWidget *widget, gpointer data)
{
    GtkTreeIter iter;
    GtkTreeIter iter2;

    if (GTK_IS_TREE_SELECTION(sel_row)
        && gtk_tree_selection_iter_is_selected(sel_row, &sel_row_iter)) {
        gtk_tree_model_get_iter_first(treemodel, &iter);

        if (!gtk_tree_selection_iter_is_selected(sel_row, &iter)) {
            while (!gtk_tree_selection_iter_is_selected(sel_row, &iter)) {
                iter2 = iter;
                gtk_tree_model_iter_next(treemodel, &iter);
            }
            iter = iter2;
            gtk_list_store_swap(list_store, &iter, &sel_row_iter);
        }
    }
}


static void config_down(GtkWidget *widget, gpointer data)
{
    GtkTreeIter iter;

    if (GTK_IS_TREE_SELECTION(sel_row)
        && gtk_tree_selection_iter_is_selected(sel_row, &sel_row_iter)) {
        iter = sel_row_iter;
        if (gtk_tree_model_iter_next(treemodel, &iter))
            gtk_list_store_swap(list_store, &iter, &sel_row_iter);
    }
}

